//
//  XSPEC12  November 2003
//
//

#include "XSFit/Fit/Fit.h"
#include "XSUser/Global/Global.h"  
#include "XSUser/Global/XSGlobal.h"  
#include "XSUtil/Utils/XSutility.h"
#include "XSUtil/Numerics/Beta.h"
#include "XSContainer.h"
#include "XSsymbol.h"
#include "XSstreams.h"
#include <sstream>
#include <iomanip>

int
XSGlobal::xsFtest(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
        Real ftest(0);
        int tclCode (TCL_OK);
        if (objc < 5)
        {
                printDocs(static_cast<char*>(cdata),"?");
                if (objc == 1 || (objc == 2 && string(Tcl_GetString(objv[1])) == "?")) 
                {
                        return globalData->autoSave(TCL_OK);
                }
                else return globalData->autoSave(TCL_ERROR);
        }
        XSutility::auto_array_ptr<Real> args (new Real[4]);
        size_t j (0);
        while (tclCode != TCL_ERROR && j < 4)
        {
                tclCode == Tcl_GetDoubleFromObj(tclInterp,objv[j+1],args.get()+j);
                ++j;
        }
        if ( j == 4)
        {
                Real& chi2 = args[0];
                Real& dof2 = args[1];
                Real& chi1 = args[2];
                Real& dof1 = args[3];
                if ( chi2 <= 0 || (dof1 - dof2) < SMALL)
                {
                        tcerr << "Bad arguments to F-Test: \n";
                        if ( chi2 <= 0 ) tcerr << "\t Chi2 = " << chi2 << '\n';
                        if (dof1 == dof2) tcerr << "\t equal degrees of freedom \n";       
                        if (dof2*dof1 == 0) tcerr << "\t  degrees of freedom are zero: "
                                        << " 1: " << dof1 << " 2: " << dof2 << '\n';     
                        return globalData->autoSave(TCL_ERROR);
                }
                Real val = (chi1 - chi2)/(dof1 - dof2)/(chi2/dof2);
                ftest = Numerics::betaI(dof2/2.,(dof1-dof2)/2.,
                                (dof2/(dof2 + (dof1 - dof2)*val)));
                Fit::lastFtest(ftest);
                tcout << " F statistic value = " << val
                      << " and probability " << ftest << std::endl;

        }
                
        return globalData->autoSave(TCL_OK);
}
